@@ -211,7 +211,8 @@ gst_gnome_vfs_src_class_init (GstGnomeVFSSrcClass * klass)
       g_param_spec_boxed ("handle",
           "GnomeVFSHandle", "Handle for GnomeVFS",
           GST_TYPE_GNOME_VFS_HANDLE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_PARAM_MUTABLE_READY | G_PARAM_READWRITE |
+          G_PARAM_STATIC_STRINGS));
 
   /* icecast stuff */
   g_object_class_install_property (gobject_class,
@@ -504,8 +505,8 @@ gst_gnome_vfs_src_received_headers_callback (gconstpointer in,
 
   for (i = in_args->headers; i; i = i->next) {
     char *data = (char *) i->data;
-    char *key = data;
     char *value = strchr (data, ':');
+    char *key;
 
     if (!value)
       continue;
@@ -624,7 +625,7 @@ gst_gnome_vfs_src_create (GstBaseSrc * basesrc, guint64 offset, guint size,
   }
 
   buf = gst_buffer_try_new_and_alloc (size);
-  if (G_UNLIKELY (buf == NULL && size == 0)) {
+  if (G_UNLIKELY (buf == NULL)) {
     GST_ERROR_OBJECT (src, "Failed to allocate %u bytes", size);
     return GST_FLOW_ERROR;
   }
