@@ -42,13 +42,6 @@
 #include "gsttcpclientsink.h"
 #include <string.h>             /* memset */
 
-/* elementfactory information */
-static const GstElementDetails gst_tcp_client_sink_details =
-GST_ELEMENT_DETAILS ("TCP client sink",
-    "Sink/Network",
-    "Send data as a client over the network via TCP",
-    "Thomas Vander Stichele <thomas at apestaart dot org>");
-
 /* TCPClientSink signals and args */
 enum
 {
@@ -131,7 +124,10 @@ gst_tcp_client_sink_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&sinktemplate));
 
-  gst_element_class_set_details (element_class, &gst_tcp_client_sink_details);
+  gst_element_class_set_details_simple (element_class,
+      "TCP client sink", "Sink/Network",
+      "Send data as a client over the network via TCP",
+      "Thomas Vander Stichele <thomas at apestaart dot org>");
 }
 
 static void
@@ -441,7 +437,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
   switch (transition) {
     case GST_STATE_CHANGE_NULL_TO_READY:
     case GST_STATE_CHANGE_READY_TO_PAUSED:
-      if (!gst_tcp_client_sink_start (GST_TCP_CLIENT_SINK (element)))
+      if (!gst_tcp_client_sink_start (sink))
         goto start_failure;
       break;
     default:
@@ -451,7 +447,7 @@ gst_tcp_client_sink_change_state (GstElement * element,
 
   switch (transition) {
     case GST_STATE_CHANGE_READY_TO_NULL:
-      gst_tcp_client_sink_stop (GST_TCP_CLIENT_SINK (element));
+      gst_tcp_client_sink_stop (sink);
     default:
       break;
   }
