@@ -85,7 +85,6 @@ vs_image_scale_linear_RGBA (const VSImage * dest, const VSImage * src,
   int y_increment;
   int x_increment;
   int y1;
-  int y2;
   int i;
   int j;
   int x;
@@ -94,19 +93,18 @@ vs_image_scale_linear_RGBA (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = dest->width * 4;
 
 #define LINE(x) ((tmpbuf) + (dest_size)*((x)&1))
 
   acc = 0;
-  y2 = -1;
   gst_videoscale_orc_resample_bilinear_u32 (LINE (0), src->pixels,
       0, x_increment, dest->width);
   y1 = 0;
@@ -123,15 +121,12 @@ vs_image_scale_linear_RGBA (const VSImage * dest, const VSImage * src,
         y1++;
       }
       if (j >= y1) {
-        gst_videoscale_orc_resample_merge_bilinear_u32 (dest->pixels +
-            i * dest->stride, LINE (j + 1), LINE (j),
-            src->pixels + (j + 1) * src->stride, (x >> 8), 0, x_increment,
-            dest->width);
+        gst_videoscale_orc_resample_bilinear_u32 (LINE (j + 1),
+            src->pixels + (j + 1) * src->stride, 0, x_increment, dest->width);
         y1++;
-      } else {
-        orc_merge_linear_u8 (dest->pixels + i * dest->stride,
-            LINE (j), LINE (j + 1), (x >> 8), dest->width * 4);
       }
+      orc_merge_linear_u8 (dest->pixels + i * dest->stride,
+          LINE (j), LINE (j + 1), (x >> 8), dest->width * 4);
     }
 
     acc += y_increment;
@@ -193,12 +188,12 @@ vs_image_scale_linear_RGB (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = dest->width * 3;
 
@@ -323,12 +318,12 @@ vs_image_scale_linear_YUYV (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = ROUND_UP_4 (dest->width * 2);
 
@@ -453,12 +448,12 @@ vs_image_scale_linear_UYVY (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = ROUND_UP_4 (dest->width * 2);
 
@@ -574,17 +569,16 @@ vs_image_scale_linear_Y (const VSImage * dest, const VSImage * src,
   int j;
   int x;
   int dest_size;
-  int xacc;
 
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = dest->width;
 
@@ -592,7 +586,6 @@ vs_image_scale_linear_Y (const VSImage * dest, const VSImage * src,
   tmp2 = tmpbuf + dest_size;
 
   acc = 0;
-  xacc = 0;
   y2 = -1;
   gst_videoscale_orc_resample_bilinear_u8 (tmp1, src->pixels,
       0, x_increment, dest->width);
@@ -607,7 +600,6 @@ vs_image_scale_linear_Y (const VSImage * dest, const VSImage * src,
       } else if (j == y2) {
         memcpy (dest->pixels + i * dest->stride, tmp2, dest_size);
       } else {
-        xacc = 0;
         gst_videoscale_orc_resample_bilinear_u8 (tmp1,
             src->pixels + j * src->stride, 0, x_increment, dest->width);
         y1 = j;
@@ -616,7 +608,6 @@ vs_image_scale_linear_Y (const VSImage * dest, const VSImage * src,
     } else {
       if (j == y1) {
         if (j + 1 != y2) {
-          xacc = 0;
           gst_videoscale_orc_resample_bilinear_u8 (tmp2,
               src->pixels + (j + 1) * src->stride, 0, x_increment, dest->width);
           y2 = j + 1;
@@ -629,7 +620,6 @@ vs_image_scale_linear_Y (const VSImage * dest, const VSImage * src,
         }
       } else if (j == y2) {
         if (j + 1 != y1) {
-          xacc = 0;
           gst_videoscale_orc_resample_bilinear_u8 (tmp1,
               src->pixels + (j + 1) * src->stride, 0, x_increment, dest->width);
           y1 = j + 1;
@@ -714,12 +704,12 @@ vs_image_scale_linear_Y16 (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = 2 * dest->width;
 
@@ -844,12 +834,12 @@ vs_image_scale_linear_RGB565 (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = dest->width * 2;
 
@@ -974,12 +964,12 @@ vs_image_scale_linear_RGB555 (const VSImage * dest, const VSImage * src,
   if (dest->height == 1)
     y_increment = 0;
   else
-    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
 
   if (dest->width == 1)
     x_increment = 0;
   else
-    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
 
   dest_size = dest->width * 2;
 
@@ -1047,3 +1037,119 @@ vs_image_scale_linear_RGB555 (const VSImage * dest, const VSImage * src,
     acc += y_increment;
   }
 }
+
+void
+vs_image_scale_nearest_AYUV64 (const VSImage * dest, const VSImage * src,
+    uint8_t * tmpbuf8)
+{
+  int acc;
+  int y_increment;
+  int x_increment;
+  int i;
+  int j;
+  int prev_j;
+
+  if (dest->height == 1)
+    y_increment = 0;
+  else
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1);
+
+  if (dest->width == 1)
+    x_increment = 0;
+  else
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1);
+
+
+  acc = 0;
+  prev_j = -1;
+  for (i = 0; i < dest->height; i++) {
+    j = acc >> 16;
+
+    if (j == prev_j) {
+      memcpy (dest->pixels + i * dest->stride,
+          dest->pixels + (i - 1) * dest->stride, dest->width * 8);
+    } else {
+      int xacc = 0;
+      vs_scanline_resample_nearest_AYUV64 (dest->pixels + i * dest->stride,
+          src->pixels + j * src->stride, src->width, dest->width, &xacc,
+          x_increment);
+    }
+
+    prev_j = j;
+    acc += y_increment;
+  }
+}
+
+void
+vs_image_scale_linear_AYUV64 (const VSImage * dest, const VSImage * src,
+    uint8_t * tmpbuf)
+{
+  int acc;
+  int y_increment;
+  int x_increment;
+  int y1;
+  int i;
+  int j;
+  int x;
+  int dest_size;
+  int xacc;
+
+  if (dest->height == 1)
+    y_increment = 0;
+  else
+    y_increment = ((src->height - 1) << 16) / (dest->height - 1) - 1;
+
+  if (dest->width == 1)
+    x_increment = 0;
+  else
+    x_increment = ((src->width - 1) << 16) / (dest->width - 1) - 1;
+
+  dest_size = dest->width * 8;
+
+#undef LINE
+#define LINE(x) ((guint16 *)((tmpbuf) + (dest_size)*((x)&1)))
+
+  acc = 0;
+  //gst_videoscale_orc_resample_bilinear_u64 (LINE (0), src->pixels,
+  //    0, x_increment, dest->width);
+  xacc = 0;
+  vs_scanline_resample_linear_AYUV64 ((guint8 *) LINE (0),
+      src->pixels, src->width, dest->width, &xacc, x_increment);
+  y1 = 0;
+  for (i = 0; i < dest->height; i++) {
+    j = acc >> 16;
+    x = acc & 0xffff;
+
+    if (x == 0) {
+      memcpy (dest->pixels + i * dest->stride, LINE (j), dest_size);
+    } else {
+      if (j > y1) {
+        xacc = 0;
+        vs_scanline_resample_linear_AYUV64 ((guint8 *) LINE (j),
+            src->pixels + j * src->stride, src->width, dest->width, &xacc,
+            x_increment);
+        //gst_videoscale_orc_resample_bilinear_u64 (LINE (j),
+        //    src->pixels + j * src->stride, 0, x_increment, dest->width);
+        y1++;
+      }
+      if (j >= y1) {
+        xacc = 0;
+        vs_scanline_resample_linear_AYUV64 ((guint8 *) LINE (j + 1),
+            src->pixels + (j + 1) * src->stride, src->width, dest->width, &xacc,
+            x_increment);
+        orc_merge_linear_u16 ((guint16 *) (dest->pixels + i * dest->stride),
+            LINE (j), LINE (j + 1), 65536 - x, x, dest->width * 4);
+        //gst_videoscale_orc_resample_merge_bilinear_u64 (dest->pixels +
+        //    i * dest->stride, LINE (j + 1), LINE (j),
+        //    src->pixels + (j + 1) * src->stride, (x >> 8), 0, x_increment,
+        //    dest->width);
+        y1++;
+      } else {
+        orc_merge_linear_u16 ((guint16 *) (dest->pixels + i * dest->stride),
+            LINE (j), LINE (j + 1), 65536 - x, x, dest->width * 4);
+      }
+    }
+
+    acc += y_increment;
+  }
+}
